package Anvil::Articles;

use Anvil::DB;
use Anvil::Common;
use POSIX;

sub
parse_date
{
	my $date = shift;
	defined $date or return time;
	my ($y,$m,$d,$H,$M) = ($date =~ /^(....)-(..)-(..)\s+(..):(..)/) or return time;
	return POSIX::mktime(0, $M, $H, $d, $m-1, $y-1900);
}

sub
load_article
{
	my $name = shift;
	my %art = ();
	$art{'id'} = $name;

	open ART, "<content/$name" or die ("Cannot open article $name");

	my $last_item;
	while (<ART>){
		chomp;
		last if /^$/;
		if (my ($cont) = /^\s+(.*)/) {
			defined $last_item or die "Parse error in header of $name";
			$art{$last_item} .= "\n$cont";
		} elsif (my ($item, $val) = /^([A-Za-z0-9-]+):\s*(.*)/) {
			$item =~ tr/A-Z/a-z/;
			$art{$item} = $val;
			$last_item = $item;
		} else {
			die "Parse error in header of article $name";
		}
	}

	while (<ART>){
		$art{'text'} .= $_;
	}

	close ART;

	my $query = "SELECT COUNT(id) AS C FROM ".
			$Anvil::Config::db_prefix.
			"_comments WHERE art ='$name';";
	my $dbh = Anvil::DB::get_dbh();
	my $sth = $dbh->prepare($query);
	$sth->execute();
	my $comments = $sth->fetchrow_hashref();
	$sth->finish();

	$art{'comments'} = $comments->{'C'};

	if (defined $Anvil::User::uid){
			$query = "SELECT COUNT(id) AS C FROM ".
				$Anvil::Config::db_prefix.
				"_comments WHERE art ='$name'
				AND date > ALL (SELECT date FROM
				".$Anvil::Config::db_prefix."_access 
				WHERE art ='$name' and uid=$Anvil::User::uid);";
	
			$sth = $dbh->prepare($query);
			$sth->execute();
			$comments = $sth->fetchrow_hashref();
			$sth->finish();

			$art{'new_comments'} = $comments->{'C'};
	}

	defined $art{'title'} or $art{'title'} = $name;
	$art{'date'} = parse_date($art{'date'});

	return bless \%art;
}

sub
heading
{
	my $art = shift;
	return "<h1><a href='" . Anvil::Common::url('read', 'art' => $art->{'id'}) . "'>" . $art->{'title'} . "</a></h1>\n";
}

sub
inline_abstract
{
	my $art = shift;
	return "" unless defined $art->{'abstract'};
	return "" if defined $art->{'separate-abstract'};
	return $art->{'abstract'} . "\n";
}

42;
