package Anvil::Comments;

use strict;
use DBI;
use Anvil::Config;
use Anvil::Common;
use Anvil::User;
use Anvil::DB;
use UCW::CGI;
use MIME::Entity;
use MIME::AltWords;
use Anvil::Articles;

sub
get_hash
{
	my $art = shift;

	my $dbh = Anvil::DB::get_dbh;	
	my $query = "SELECT id, art, subject, text, parent, name, email, UNIX_TIMESTAMP(date) AS date FROM ".
                        $Anvil::Config::db_prefix.
                        "_comments WHERE art ='$art';";
        my $sth = $dbh->prepare($query);
        $sth->execute();
        my $comments;

	my %tree = ();
	my %cmt = ( 0 => { "id" => 0 } );

	while ($comments = $sth->fetchrow_hashref()){
		push @{$tree{$comments->{'parent'}}}, $comments;	
		$cmt{$comments->{'id'}} = $comments;
	}

	$sth->finish();

	for my $id (keys %tree){
		$cmt{$id}->{'sons'} = $tree{$id};
	}

	return \%cmt;
}

sub
get_color
{
	my $n = shift;
	my $start = $Anvil::Config::cmt_start_color; 
	my $end = $Anvil::Config::cmt_end_color; 
	my $depth = $Anvil::Config::cmt_color_depth; 

	my ($r1, $g1, $b1) = map hex, ($start =~ /(..)(..)(..)/);
	my ($r2, $g2, $b2) = map hex, ($end =~ /(..)(..)(..)/);

	my $alpha = $n / $depth;

	my $r = int($r1 * (1 - $alpha) + $r2 * ($alpha));
	my $g = int($g1 * (1 - $alpha) + $g2 * ($alpha));
	my $b = int($b1 * (1 - $alpha) + $b2 * ($alpha));

	return sprintf "%02x%02x%02x", $r, $g, $b; 

}

sub
get_ranks
{
	my ($cmt, $id) = @_; 
	my $tree_size = 1;
	my @subtrees = (); 

	for my $t (@{$cmt->{$id}->{'sons'}}){
		my ($subtree_size, $subtree_people, $subtree_num_people) = 
			get_ranks($cmt, $t->{'id'});
		$tree_size += $subtree_size;
		push @subtrees, [$subtree_people, $subtree_num_people];
	}

	my $maxlen = 0;
	my $maxref;
	for my $k (@subtrees){
		my ($subtree_people, $subtree_num_people) = @{$k};
		if ($subtree_num_people >= $maxlen){
			$maxlen = $subtree_num_people;
			$maxref = $subtree_people;
		}
	}

	my $people;
	my $num_people;
	if (not defined $maxref){
		$people = {};
		$num_people = 0;
	} else {
		$people = $maxref; 
		$num_people = $maxlen;
		for my $k (@subtrees){
			next if $maxref eq $k;
			my ($subtree_people, $subtree_num_people) = @{$k};
			for my $l (keys %{$subtree_people}){
				if (not defined $people->{$l}){
					$people->{$l} = 1;
					$num_people++;
				}
			}
		}

	}
	my $t = $cmt->{$id};
	if ($id and not defined $people->{$t->{'name'}}){
			$people->{$t->{'name'}} = 1;
			$num_people++;
	}

	$cmt->{$id}->{'rank'} = $num_people / $tree_size;
	$cmt->{$id}->{'tree_size'} = $tree_size;

	return ($tree_size, $people, $num_people);
}

sub
pc
{
	my ($ind, $id, $cmt, $lastseen, $recurse) = @_;
	my $hidden = 0;

	if ($id){
		my $u = $cmt->{$id};
		if ($recurse && ($u->{'rank'} < $Anvil::Config::rank_treshold)){
			$hidden = 1;
		}

		my $new = $lastseen<$u->{'date'}?' <span class="new_comm">NOVÝ</span>':'';
		my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($u->{'date'});

		print "<div class =\"comment\" style=\"margin-left:".
			($ind*3).";
			background-color: #".get_color($ind).";
			padding: 2px;
			padding-top: 10px;\">\n";


		print "<span class=\"subject\">".UCW::CGI::html_escape($u->{'subject'})."</span>\n";
		print "<span class=\"name\">".UCW::CGI::html_escape($u->{'name'})."</span>\n";
		printf "<span class=\"date\"> (%d. %d. %d - %d:%02d%s)</span>\n", $mday, $mon+1, ($year+1900), $hour, $min, $new;
		if ($recurse){
			print "<span class=\"comment_toggle\">\n
				<a href=\"javascript:toggle('$u->{'id'}')\">";
				print "<span id=\"r$u->{'id'}\"".($hidden?'':'style="display: none;"').">Rozbalit</span>";
				print "<span id=\"s$u->{'id'}\"".($hidden?'style="display: none;"':'').">Sbalit</span>";
			print "($u->{'tree_size'})";
			print "</a>\n </span>\n";
		}

		print "<br>\n";

		print "<div id=\"x$u->{'id'}\"". #we cannot start with number
			($hidden?' style="display: none;"':'').">\n";
		$u->{'text'} =~ s/\r//gm;
		$u->{'text'} =~ s/\n/<br>/gm;
		print "<div class=\"comment_text\">";
		print $u->{'text'};
		print "</div>\n";

		if ($recurse && defined $Anvil::User::uid){
			print "<div class=\"get_comment_link\">";
			print "<a href='", Anvil::Common::url('send', 'send' => $u->{'id'}), "'>Zaslat komentář</a>";
			print "</div>\n";
		}

		print "<div class='reply_link'>";
		print "<a href='", Anvil::Common::url('reply', 'reply_to' => $u->{'id'}), "'>Odpovědět</a>";
		print "</div>\n";
	}

	if ($recurse){
		for my $u (@{$cmt->{$id}->{'sons'}}){
			pc($ind+1, $u->{'id'}, $cmt, $lastseen, $recurse);
		}
	}

	if ($id){
		print "</div>\n";
		print "</div>\n";
	}
}

sub
print_comment
{
	my ($cmt, $id) = @_;
	pc(0, $id, $cmt, time() + 1000, 0);
}

sub
print_comments
{
	my ($art, $cmt) = @_;
	my $lastseen; 
	my $time = time();

	#get lastseen info
	my ($dbh, $query, $sth);
	if (defined $Anvil::User::uid){
		$dbh = Anvil::DB::get_dbh;
		$query = "SELECT UNIX_TIMESTAMP(date) AS date FROM ".
			$Anvil::Config::db_prefix."_access WHERE uid="
			.$Anvil::User::uid." AND art='$art';";
		$sth = $dbh->prepare($query);
		$sth->execute();
		my $res = $sth->fetchrow_hashref();
		$sth->finish;
		if (not defined $res->{'date'}){
			$query = "INSERT INTO ".$Anvil::Config::db_prefix.
				"_access (uid, art, date) VALUES (".
				$Anvil::User::uid.",'$art',FROM_UNIXTIME(".$time."));";
				$dbh->do($query);
				$lastseen = $time;
		} else {
			$lastseen = $res->{'date'};
		}
	} else {
		$lastseen = $time;
	}
	
	#prepare hiding javascript
	#learned it here: http://www.webmasterworld.com/forum91/441.htm
	print <<EOF
<script language=javascript type='text/javascript'>

function toggle(id){
	if (document.getElementById){
		obj = document.getElementById("x" + id);
		r = document.getElementById("r" + id);
		s = document.getElementById("s" + id);
		if (obj.style.display == "none"){
			obj.style.display = "";
			s.style.display = "";
			r.style.display = "none";
		} else {
			obj.style.display = "none";
			s.style.display = "none";
			r.style.display = "";
		}
	}
} 

</script> 	
EOF
;
	
	#print comments
	get_ranks($cmt, 0); 
	pc(0, 0, $cmt, $lastseen, 1);
	
	#new comment link
	print "<div class='reply_link'><a href='",
		Anvil::Common::url('reply', 'reply_to' => 0),
		"'>Nový komentář</a></div>\n";

	#update lastseen info
	if (defined $Anvil::User::uid){
		$query = "UPDATE ".$Anvil::Config::db_prefix.
		"_access SET date=FROM_UNIXTIME(".time.
		") WHERE uid=".$Anvil::User::uid." AND art='$art';";
		$dbh->do($query);
	}
	return $cmt;

}

sub
collect_ref_ids
{
	my $id = shift;
	my $cmt = shift;
	my $cur_parent = $cmt->{$id}->{'parent'};
	my @res;

	while ($cur_parent) {
		push @res, $cur_parent;
		$cur_parent = $cmt->{$cur_parent}->{'parent'};
	}
	push @res, 0;

	return \@res;
}

sub
collect_emails
{
	my ($cmt, $id) = @_;
	my $res; 

	if (not defined $id){
		return $res;
	}

	my $parent = $cmt->{$id}->{'parent'}; 
	$res = collect_emails($cmt, $parent) if $parent;

	push @{$res}, $cmt->{$id}->{'email'} 
		if defined $cmt->{$id}->{'email'};
	return $res;
}

sub
find_receivers
{
	my ($id, $art, $cmt) = @_;
	my @receivers;

	#people who want everything
	my $dbh = Anvil::DB::get_dbh();
	my $query = "SELECT email FROM ".$Anvil::Config::db_prefix."_users
		WHERE set_allnew=1";
	my $sth = $dbh->prepare($query);
	$sth->execute();
	my $ref;
	while ($ref = $sth->fetchrow_hashref()){
		push @receivers, $ref->{'email'};
	}
	$sth->finish();

	#people who want replies
	my $colref = collect_emails($cmt,$id);
	my @collected = (defined $colref?@{$colref}:());	
	if (@collected){
		my $allreps = join ",", map { $dbh->quote($_) } @collected;
		$query = "SELECT email FROM ".$Anvil::Config::db_prefix."_users
			WHERE set_allrep=1 AND email in ($allreps)";

		$sth = $dbh->prepare($query);
		$sth->execute();
		while ($ref = $sth->fetchrow_hashref()){
			push @receivers, $ref->{'email'};
		}
		$sth->finish();
	}

	#people who want all comments under articles they commented
	$query = "SELECT email FROM ".$Anvil::Config::db_prefix."_users
		WHERE set_allmy=1 AND email in 
		(SELECT email from ".$Anvil::Config::db_prefix."_comments
			WHERE art='$art');";

	$sth = $dbh->prepare($query);
	$sth->execute();
	while ($ref = $sth->fetchrow_hashref()){
		push @receivers, $ref->{'email'};
	}
	$sth->finish();

	my %rec = map {$_ => undef} @receivers;
	return keys %rec; 
}

sub
html_deescape
{
	my $in = shift;

	$in =~ s/&amp;/&/g;
	$in =~ s/&lt;/</g;
	$in =~ s/&gt;/>/g;
        $in =~ s/&quot;/"/g;

        return $in;
}

sub
save_comment
{
	my ($art,$name,$subject,$parent,$email,$text) = @_;

	my $query = "INSERT INTO ".$Anvil::Config::db_prefix."_comments 
	(art, name, subject, date, parent, email, text) 
	values (?, ?, ?, FROM_UNIXTIME(?), ?, ?, ?);";

	my $dbh = Anvil::DB::get_dbh();
	my $sth = $dbh->prepare($query);
	$sth->execute($art,$name,$subject,time(),$parent,$email,$text) 
		or return 0; 
	$sth->finish;

	#send comment to the people who should get it
	#first we are wondering about id that our comment got
	$query = "SELECT LAST_INSERT_ID() AS id;";
	$sth = $dbh->prepare($query);
	$sth->execute();
	my $id = $sth->fetchrow_hashref();
	$id = $id->{'id'};

	#get list of references
	my @receivers;
	my ($references, $reply);
	#FIXME - we do not need all the data here
	my $cmt = Anvil::Comments::get_hash($art);
	my @references = map {s/(\d+)/<$1-$art\@$Anvil::Config::mail_domain>/;$_} @{Anvil::Comments::collect_ref_ids($id, $cmt)};
	$references = join (' ', @references);
	$reply = $references[0];

	@receivers = find_receivers($id, $art, $cmt);

	my $send = MIME::Entity->build(
                                From => "$name <$Anvil::Config::mail_from_email>", 
                                References => $references,
				'Message-Id' => "<$id-$art\@$Anvil::Config::mail_domain>",
                                'In-Reply-To:' => $reply,
				Subject => MIME::AltWords::encode_mimewords($subject, 'Charset' => 'utf-8'), 
				Data => html_deescape($text),
				Charset => 'utf-8',
                                Encoding => 'base64',
				'MIME-Version' => '1.0',
				Type => "text/plain",
			);
	my $n = 0;
	while ($n < @receivers){
		my $to = join " ", @receivers[$n..List::Util::min($n+9,$#receivers)];
		$n += 10;

		open SEND, "|/usr/lib/sendmail -f $Anvil::Config::mail_from_email \"$to\"" or return 0; 
		$send->print(\*SEND);
		close SEND;
	}

	return 1;
}

42;
