#!/usr/bin/perl -w

use strict;

use Anvil::Config;
use Anvil::Common;
use Anvil::DB;
use Anvil::Articles;
use UCW::CGI;

my $debug = 0;

### Load all known articles ###

my $path = $Anvil::Config::dir_path;
chdir $path or die("Cannot chdir to $path");

opendir DIR, "content" or die("Cannot open content directory: $!");
my @articles = sort { $b->{'date'} <=> $a->{'date'} }
	       map { Anvil::Articles::load_article($_) }
	       grep { !/^\./ && !/~$/ && -f "content/$_"}
	       readdir DIR;
closedir DIR;

@articles = grep {!$_->{'skip-index'}} @articles;

my $art;
my $art_hr;

### Generate indices ###

my $dbh = Anvil::DB::get_dbh;

my %known_arts = ();
my $sth = $dbh->prepare("SELECT art,UNIX_TIMESTAMP(date) AS date FROM " . Anvil::DB::table_name("articles"));
$sth->execute();
while (my $r = $sth->fetchrow_hashref) {
	$known_arts{$r->{'art'}} = $r->{'date'};
	print "Known: ", $r->{'art'}, " (", $r->{'date'}, ")\n" if $debug;
}
$sth->finish();

my $sth_ins = $dbh->prepare("INSERT INTO " . Anvil::DB::table_name("articles") . "(art,date) VALUES (?,FROM_UNIXTIME(?))");
my $sth_upd = $dbh->prepare("UPDATE " . Anvil::DB::table_name("articles") . " SET date=FROM_UNIXTIME(?) WHERE art=?");
for $art_hr (@articles) {
	$art = $art_hr->{'id'};
	if (!defined($known_arts{$art})) {
		print "Inserting $art\n" if $debug;
		$sth_ins->execute($art, $art_hr->{'date'});
	} elsif ($known_arts{$art} != $art_hr->{'date'}) {
		print "Updating $art\n" if $debug;
		$sth_upd->execute($art_hr->{'date'}, $art);
	}
	delete $known_arts{$art};
}
$sth_upd->finish();
$sth_ins->finish();

my $sth_del = $dbh->prepare("DELETE FROM " . Anvil::DB::table_name("articles") . " WHERE art=?");
for $art (keys %known_arts) {
	print "Deleting $art\n" if $debug;
	$sth_del->execute($art);
}
$sth_del->finish();

### Generate RSS ###

open RSS, ">rss.xml.new" or die("Cannot create rss.xml.new: $!");
my $root = Anvil::Common::url('index', 'absolute' => 1);
my $rss = Anvil::Common::url('rss', 'absolute' => 1);
print RSS <<EOF
<?xml version="1.0" encoding="utf-8"?>
<rss version="2.0" xmlns:atom="http://www.w3.org/2005/Atom">
<channel>
<title>$Anvil::Config::web_name</title>
<link>$root</link>
<description>$Anvil::Config::web_name</description>
<atom:link href="$rss" rel="self" type="application/rss+xml" />
EOF
;
for $art_hr (@articles[0..9]) {
	defined $art_hr or next;
	my $text = $art_hr->inline_abstract . $art_hr->{'text'};
	$text = UCW::CGI::html_escape($text);
	my $url = Anvil::Common::url('read', 'art' => $art_hr->{'id'}, 'absolute' => 1);
	print RSS <<EOF
<item>
<title>$art_hr->{'title'}</title>
<link>$url</link>
<description>
$text
</description>
<guid isPermaLink="false">$art_hr->{'id'}</guid>
</item>
EOF
;
	
}
print RSS "</channel>\n </rss>\n";
close RSS;
rename "rss.xml.new", "rss.xml" or die "Cannot install new rss.xml: $!";
