#!/usr/bin/perl -w

use strict;
use Anvil::Config;
use Anvil::Comments;
use Anvil::DB;
use MIME::Parser;
use MIME::Entity;
use MIME::AltWords;
use Mail::Header;
use Email::Address;
use DBI;
use UCW::CGI;
use List::Util;
use Encode;

sub
report_bug
{
	my $death = shift;
	my $ent = shift;

	my $to = $ent->get('From');
	$to = $ent->get('Reply-To') if defined $ent->get('Reply-To');
	
	open SEND, "|/usr/lib/sendmail -f $Anvil::Config::mail_bug_email \"$to\"" or die("Cannot open sendmail");

	my $reply = MIME::Entity->build(
				From => $Anvil::Config::mail_bug_email,
				To => $to,
				Subject => $death,
				References => $ent->get('Message-ID'),
				'In-Reply-To:' => $ent->get('Message-ID'),
				'Data' => $death."\n Please try again :-)\n",
			);
	
	$reply->print(\*SEND);
	close SEND;
	exit;
}

my $parser = new MIME::Parser;
$parser->output_under("/tmp");
my $ent = $parser->parse(\*STDIN);

my ($name, $email, $text, $parent, $subject, $art);
my $dbh = Anvil::DB::get_dbh(); 

report_bug("No subject header", $ent) unless defined $ent->get('Subject'); 
$subject =  MIME::AltWords::decode_mimewords($ent->get('Subject'), 
				('Charset' => 'utf-8'));
chomp $subject;

$ent->combine('References');
my @ref = split /\s+/, $ent->get('References');
if (defined $ent->get('In-Reply-To')){
	push @ref, $ent->get('In-Reply-To');
}
map chomp, @ref;
$parent = -1;
for my $r (@ref) {
	my ($p, $a) = $r =~ /<(\d+)-([A-Za-z0-9_\/-]+)\@$Anvil::Config::mail_domain>/;
	if (defined $p and defined $a and $p > $parent){
		($parent, $art) = ($p, $a);
	}
}
report_bug("Cannot parse references: parent not defined", $ent) if $parent == -1;
report_bug("Cannot parse references: article not defined", $ent) unless defined $art;

($email) = Email::Address->parse($ent->get('From'), $ent);

my $query = "SELECT name FROM ".$Anvil::Config::db_prefix."_users WHERE
	email = ?;";
my $sth = $dbh->prepare($query);
$sth->execute($email->address);
my $res = $sth->fetchrow_hashref(); 

if (defined $res->{'name'}) {
	$name = $res->{'name'};
} elsif (defined $email->name){
	$name = MIME::AltWords::decode_mimewords($email->name, 'Charset' => 'utf-8');
} else {
	$name = $email->user;
}
$email = $email->address;
report_bug("Cannot parse username", $ent) if not defined $name or not defined $email;

my $orig = $ent; #HACK
if (not defined $ent->bodyhandle){
	for my $part ($ent->parts){
		if ($part->get('Content-Type') =~ /text\/plain/){
			$ent = $part;
			last;
		}
	}
}

report_bug("Cannot find any text/plain part", $orig) unless defined $ent->bodyhandle;

my $body = $ent->bodyhandle;
my $charset = $ent->get('Content-Type');
$charset =~ s/.*;\s+charset="{0,1}([^;"]*)"{0,1}.*/$1/m;
$text = $body->as_string;

$text = decode($charset,$text);
$text = encode("utf-8", $text);

$text = UCW::CGI::html_escape($text);

if ($parent){
	$query = "SELECT art FROM ".$Anvil::Config::db_prefix.
		"_comments WHERE id = $parent";
	$sth = $dbh->prepare($query);
	$sth->execute();
	my $check = $sth->fetchrow_hashref();
	report_bug("Article/comment id mismatch", $orig) 
		unless (defined $check->{'art'} and $check->{'art'} eq $art);
}

#save comment
Anvil::Comments::save_comment($art,$name,$subject,$parent,$email,$text) or report_bug("Cannot save comment");
